
%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% Generates figures 2 and 4.
% 
% William Gatt
% 
% The data files used in this script need to be generated using 'run_OBC_simulations.m'
% ---------------------------------------------------------------------------------

%% Stores only the most relevant data from the simulation
% clc
warning off
set(groot,'defaultlinelinewidth',1);
set(groot,'defaultAxesXGrid','off');
set(groot,'defaultAxesYGrid','off');
set(groot, 'defaultAxesTickLabelInterpreter','latex')
set(groot, 'defaultLegendInterpreter','latex');
set(groot, 'defaultTextInterpreter','latex')

addpath('functions')
save_fig = 0; % Saves figure 
my_colours1;

%% --------------------------------------------------------------------------------

fprintf('\nLoading data...\n\n');
curdir = pwd;

cd('New_Results');
load('simulation_dmH_0.75.mat')
cd(curdir)

kink = delta_mHv./delta_mL;

plow = 10;
phigh = 90;

% Set.num_del = Set.num_del + 2;
Set.num_del = 41;

%Pre-allocate
var_q = NaN(Set.num_del,1);
var_Y = NaN(Set.num_del,1);
var_B4Y = NaN(Set.num_del,1);
var_m = NaN(Set.num_del,1);
var_mju = NaN(Set.num_del,1);
var_Cb = NaN(Set.num_del,1);
var_Cs = NaN(Set.num_del,1);
var_PI4 = NaN(Set.num_del,1);

var_qMax = NaN(Set.num_del,1);
var_YMax = NaN(Set.num_del,1);
var_B4YMax = NaN(Set.num_del,1);
var_mMax = NaN(Set.num_del,1);
var_mjuMax = NaN(Set.num_del,1);
var_CbMax = NaN(Set.num_del,1);
var_CsMax = NaN(Set.num_del,1);
var_PI4Max = NaN(Set.num_del,1);
var_qMin = NaN(Set.num_del,1);
var_YMin = NaN(Set.num_del,1);
var_B4YMin = NaN(Set.num_del,1);
var_mMin = NaN(Set.num_del,1);
var_mjuMin = NaN(Set.num_del,1);
var_CbMin = NaN(Set.num_del,1);
var_CsMin = NaN(Set.num_del,1);
var_PI4Min = NaN(Set.num_del,1);

Regimes(1,:) = [mean(Set.regime_count')]/4;

%% MAIN LOOP
delta_m_sym = delta_m;
max_kink = 20;
delta_mHv = linspace(delta_m_sym,delta_m_sym*max_kink,39)';

range2 = [15.00:0.5:35.0]';

delta_mHv1 = [1:0.50:50.0]';

kink = delta_mHv1./delta_m_sym';

for delmH = 1:length(delta_mHv1)
    delta_mH_temp = delta_mHv1(delmH);
    disp(delta_mH_temp);
    cd(strcat(pwd,'/New_Results'));
    load_string  = sprintf('simulation_dmH_%2.2f.mat',delta_mH_temp);
    load(load_string);
    cd(curdir)
      
        var_Y(delmH,1)   = median(var(Y_p));
        var_B4Y(delmH,1) = median(var(B_4Y_p));
        var_m(delmH,1)   = median(var(m_p));
        var_PI4(delmH,1) = median(var(PI4_p));
        
        var_B4Y_mean(delmH,1) = mean(var(B_4Y_p));
        var_m_mean(delmH,1)   = mean(var(m_p));
        
        var_YMax(delmH,1)   = prctile(var(Y_p),phigh);        
        var_B4YMax(delmH,1) = prctile(var(B_4Y_p),phigh);  
        var_mMax(delmH,1)   = prctile(var(m_p),phigh);  
        var_PI4Max(delmH,1) = prctile(var(PI4_p),phigh);  
        
        var_YMin(delmH,1)   = prctile(var(Y_p),plow);        
        var_B4YMin(delmH,1) = prctile(var(B_4Y_p),plow);  
        var_mMin(delmH,1)   = prctile(var(m_p),plow);  
        var_PI4Min(delmH,1) = prctile(var(PI4_p),plow); 
            
        Regimes(delmH,:) = [mean(Set.regime_count')]/4;

end
fprintf('\n....Done!\n\n\n');


%% FIGURE - VOLATILITY

figure('Name','Volatility','PaperPositionMode','auto',...
    'PaperOrientation','portrait',...     
    'PaperSize',[25 15],'Color','White');
% 'PaperSize',[10 6],'Color','White');

time = [1:length(var_B4Y)]';
nn  =   length(delta_mHv1);
porder = 3;

xaxis = kink;

%----OUTPUT-----
rvar_Y = var_Y./var_Y(1);
rvar_YMax = var_YMax./var_YMax(1);
rvar_YMin = var_YMin./var_YMin(1);
[Vminrvar_Y, Iminrvar_Y] = min(rvar_Y);

subplot(2,2,1); grid off; hold on; box on;
title('Output','fontsize',14);
plot(xaxis,rvar_Y(1:nn),'Linestyle','none','color',red,'Marker','o','MarkerFaceColor',red, 'MarkerSize',3);
plot(xaxis(Iminrvar_Y),Vminrvar_Y,'Linestyle','none','color','k','Marker','d','MarkerFaceColor','k', 'MarkerSize',7);
xlim([min(kink) max(kink)])
set(gca,'Xtick',[1 10:10:60])
xlim([0 65])
ylim([0.5 1.0])
set(gca,'Ytick',[0.5:0.1:1])
xlabel('$\kappa$','interpreter','latex','fontsize',12);
ylabel('Relative variance','fontsize',14)

%----INFLATION----
rvar_PI4 = var_PI4./var_PI4(1); 
rvar_PI4Max = var_PI4Max./var_PI4Max(1);
rvar_PI4Min = var_PI4Min./var_PI4Min(1);
[Vminrvar_PI4, Iminrvar_PI4] = min(rvar_PI4);

subplot(2,2,2); grid off; hold on; box on;
title('Inflation','fontsize',14);
plot(xaxis,rvar_PI4(1:nn),'Linestyle','none','color',red,'Marker','o','MarkerFaceColor',red, 'MarkerSize',3);
plot(xaxis(Iminrvar_PI4),Vminrvar_PI4,'Linestyle','none','color','k','Marker','d','MarkerFaceColor','k', 'MarkerSize',7);
xlim([min(kink) max(kink)])
set(gca,'Xtick',[1 10:10:60])
xlim([0 65])
set(gca,'Ytick',[0.4:0.1:1])
ylim([0.4 max(ylim)])
xlabel('$\kappa$','interpreter','latex','fontsize',12);
ylabel('Relative variance','fontsize',14)


%----CREDIT TO GDP----
rvar_B4Y = var_B4Y./var_B4Y(1); 
rvar_B4YMax = var_B4YMax./var_B4YMax(1); 
rvar_B4YMin = var_B4YMin./var_B4YMin(1); 
[Vminrvar_B4Y, Iminrvar_B4Y] = min(rvar_B4Y);

subplot(2,2,3); grid off; hold on; box on;
title('Credit to output','fontsize',14);
plot(xaxis,rvar_B4Y(1:nn),'Linestyle','none','color',red,'Marker','o','MarkerFaceColor',red, 'MarkerSize',3);
plot(xaxis(Iminrvar_B4Y),Vminrvar_B4Y,'Linestyle','none','color','k','Marker','d','MarkerFaceColor','k', 'MarkerSize',7);
xlim([min(kink) max(kink)])
set(gca,'Xtick',[1 10:10:60])
xlim([0 65])
set(gca,'Ytick',[0.4:0.1:1])
ylim([0.4 max(ylim)])
xlabel('$\kappa$','interpreter','latex','fontsize',12);
ylabel('Relative variance','fontsize',14)

%----LTV RATIO----
rvar_m = var_m./var_m(1); 
rvar_mMax = var_mMax./var_mMax(1); 
rvar_mMin = var_mMin./var_mMin(1); 
[Vminrvar_m, Iminrvar_m] = min(rvar_m);

subplot(2,2,4); grid off; hold on; box on;
title('LTV ratio','fontsize',14);
plot(xaxis,rvar_m(1:nn),'Linestyle','none','color',red,'Marker','o','MarkerFaceColor',red, 'MarkerSize',3);
plot(xaxis(Iminrvar_m),Vminrvar_m,'Linestyle','none','color','k','Marker','d','MarkerFaceColor','k', 'MarkerSize',7);

xlim([min(kink) max(kink)])
set(gca,'Xtick',[1 10:10:60])
xlim([0 65])
ylim([1 1.7])
set(gca,'Ytick',[1:0.1:1.7])

xlabel('$\kappa$','interpreter','latex','fontsize',12);
ylabel('Relative variance','fontsize',14)

if  save_fig == 1
    print(gcf,'-dpdf','-fillpage','../Figures/Fig_relVariance.pdf');
end


%% FIGURE - REGIME FREQUENCY

parent1=figure('Name','Volatility','PaperPositionMode','auto',...
    'PaperOrientation','portrait',...
    'PaperSize',[25 15],'Color','White');
% 'PaperSize',[10 6],'Color','White');

% strings_top  = {'Binding','Slack'};
% strings_side = {'Credit boom','Credit bust'};

strings = {'Regime 1', 'Regime 2', 'Regime 3', 'Regime 4'};

for j=1:4
    subplot(2,2,j); 
    plot(kink, Regimes(:,j),'Linestyle','none','color',red,'Marker','o','MarkerFaceColor',red, 'MarkerSize',3);
%     xlim([min(kink) max(kink)])
    set(gca,'Xtick',[1 10:10:60])
    xlim([0 65])
    xlabel('$\kappa$','interpreter','latex','fontsize',12);
    ytickformat('%2.0f\\%%');
    title(strings(j),'fontsize',12)
    switch j
        case 1
            ylim([40 65]);
        case 4
            ylim([0.0 0.4]);
            ytickformat('%1.1f\\%%')
    end
end

text(-101,0.65,'Credit boom','FontSize',14,'Rotation',90);
text(-101,0.10,'Credit bust','FontSize',14,'Rotation',90);
text(-78.0,1.05,'Borrowing constraint binds','FontSize',14,'Rotation',0);
text(08,1.05,'Borrowing constraint slack','FontSize',14,'Rotation',0);

if  save_fig == 1
    print(gcf,'-dpdf','-fillpage','../Figures/Fig_regimefrequency.pdf');
end
return

%% LOSS FUNCTION
Loss_credit = var_B4Y_mean;
Loss_LTV = var_m_mean;
weight_credit = 1.0;
weight_LTV = [0.0:0.1:0.4];
Loss = weight_credit.*Loss_credit + weight_LTV.*Loss_LTV;

figure;
plot(xaxis,Loss,'Linestyle','-','Marker','o','MarkerFaceColor',red, 'MarkerSize',4);
xlabel('$\kappa$','interpreter','latex','fontsize',12);
ylabel('Loss')
legend(num2str(weight_LTV'))


