%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% Distribution of output and inflation for G@R analysis
% 
% William Gatt
% 
% The data files have to be generated first using 'run_OBC_simulations.m'


clear all
% clc
set(groot, 'defaultAxesTickLabelInterpreter','latex')
set(groot, 'defaultLegendInterpreter','latex');
set(groot, 'defaultTextInterpreter','latex')
    addpath('functions')
    
    my_colours1
    col_vec1 = [green; red; blue];
    
    % Save figure as PDF
    save_fig = 0;
    lw = 2;
    
    nm = [1:300];
    np = [1:400];
    
    dom =[-0.03:0.00001:0.03];  % domain in percentage terms

    d_values = ["0.00";"0.75"; "32.50"];
    
    n = size(d_values,1);

    Y_p_stacked = NaN(np(end)*nm(end),n);
    PI_Y4_p_stacked = NaN(np(end)*nm(end),n);
    PI4_p_stacked = NaN(np(end)*nm(end),n);
    
    kappa = NaN(n,1);
    
%% Main loop
    for j = 1 : n
        str = strcat('New_Results\simulation_dmH_',d_values(j),'_long.mat');
%         str = strcat('New_Results\simulation_dmH_',d_values(j),'.mat');
        load(str);

        % GDP
        Y_p_stacked(:,j) = reshape(Y_p(np,nm),Set.nshckperiods*Set.numiter,1); % stack all instances into one long vector
        pd1(:,j) = fitdist(Y_p_stacked(:,j),'kernel');
        kdensity1(:,j) = pdf(pd1(:,j),dom);
        
        %GDP Growth
        PI_Y4_p_stacked(:,j) = reshape(PI_Y4_p(np,nm),Set.nshckperiods*Set.numiter,1); % stack all instances into one long vector
        pd2(:,j) = fitdist(PI_Y4_p_stacked(:,j),'kernel');
        kdensity2(:,j) = pdf(pd2(:,j),dom);
        
        %INFLATION
        PI4_p_stacked(:,j) = reshape(PI4_p(np,nm),Set.nshckperiods*Set.numiter,1); % stack all instances into one long vector
        pd3(:,j) = fitdist(PI4_p_stacked(:,j),'kernel');
        kdensity3(:,j) = pdf(pd3(:,j),dom);
        
        kappa(j,1) = str2double(d_values(j))/0.75;
    end
    
    stats.kappa = kappa';
    
    stats.Y_mean = mean(Y_p_stacked*100);
    stats.Y_p5  = prctile(Y_p_stacked*100,05);
    stats.Y_p25 = prctile(Y_p_stacked*100,25);
    stats.Y_p50 = prctile(Y_p_stacked*100,50);
    stats.Y_p75 = prctile(Y_p_stacked*100,75);
    stats.Y_p95 = prctile(Y_p_stacked*100,95);
    
    stats.PI_Y_mean = mean(PI_Y4_p_stacked*100);
    stats.PI_Y_p5  = prctile(PI_Y4_p_stacked*100,05);
    stats.PI_Y_p25 = prctile(PI_Y4_p_stacked*100,25);
    stats.PI_Y_p50 = prctile(PI_Y4_p_stacked*100,50);
    stats.PI_Y_p75 = prctile(PI_Y4_p_stacked*100,75);
    stats.PI_Y_p95 = prctile(PI_Y4_p_stacked*100,95);
    
    stats.PI_mean = mean(PI4_p_stacked*100);
    stats.PI_p5  = prctile(PI4_p_stacked*100,05);
    stats.PI_p25 = prctile(PI4_p_stacked*100,25);
    stats.PI_p50 = prctile(PI4_p_stacked*100,50);
    stats.PI_p75 = prctile(PI4_p_stacked*100,75);
    stats.PI_p95 = prctile(PI4_p_stacked*100,95);
    
    [val,ind1] = max(kdensity1);
    [val,ind2] = max(kdensity2);
    [val,ind3] = max(kdensity3);
    stats.maxprob1 = dom(ind1)*100;
    stats.maxprob2 = dom(ind2)*100;
    stats.maxprob3 = dom(ind3)*100;
     
    
%% OUTPUT

   figure('Name','Kernel density - output','PaperPositionMode','auto',...
          'PaperOrientation','landscape',...
        'PaperSize',[20 12],'Color',[1 1 1])
    
    
    
    for j = 1 : n        
        h = plot(dom*100,kdensity1(:,j)/1e4,'LineWidth',lw,'Color', [col_vec1(j,:)]);
        hold on;
        k.l(j) = h;
    end
    
    grid off
    set(k.l(1),'LineStyle','--');
    set(k.l(2),'LineStyle','-.');
    
    % 5th percentile
    for j = 1: n
        l(j)=line([stats.Y_p5(j) stats.Y_p5(j)],[0 max(ylim)],'Color',col_vec1(j,:),'LineWidth',1);
    end
    
    line([0 0],[0 max(ylim)],'Color',grey,'LineWidth',0.7)
    
    set(l(1),'LineStyle','--');
    set(l(2),'LineStyle','-.');

    ylabel('Kernel density');
    xlabel('Output - \% deviations from steady state');
        
    leg1=legend([k.l(1),k.l(2),k.l(3)],...
            'Fixed LTV',...
            'Symmetric policy',...
            'Asymmetric policy',...
            'FontSize',10,'Location','NE','NumColumns',1);
        
    set(gca,'YColor','k')
    set(gca,'XColor','k')
    set(gca,'FontSize',12)
    xlim([-2.0 2])
    xtickformat('%2.1f');

if  save_fig == 1
    print(gcf,'-dpdf','-fillpage','../Figures/Fig_GAR_output.pdf');
end


%% INFLATION

figure('Name','Kernel density - inflation','PaperPositionMode','auto',...
     'PaperOrientation','landscape',...
        'PaperSize',[20 12],'Color',[1 1 1])
%           'PaperSize',[8 5],'Color',[1 1 1])    
  
    lw = 2;
           
    % INFLATION density
    for j = 1 : n        
        h = plot(dom*100,kdensity3(:,j)/1e4,'LineWidth',lw,'Color', [col_vec1(j,:)]);
        hold on;
        k.l(j) = h;
    end
    
    grid off
    set(k.l(1),'LineStyle','--');
    set(k.l(2),'LineStyle','-.');
    
%     5th percentile
    for j = 1: n
        l(j)=line([stats.PI_p5(j) stats.PI_p5(j)],[0 max(ylim)],'Color',col_vec1(j,:),'LineWidth',1);
    end
    
    line([0 0],[0 max(ylim)],'Color',grey,'LineWidth',0.7)
    
    set(l(1),'LineStyle','--');
    set(l(2),'LineStyle','-.');

    ylabel('Kernel density');
    xlabel('Inflation - ppt. deviations from steady state');
        
    leg1=legend([k.l(1),k.l(2),k.l(3)],...
            'Fixed LTV',...
            'Symmetric policy',...
            'Asymmetric policy',...
            'FontSize',10,'Location','NE','NumColumns',1);
    
    set(gca,'YColor','k')
    set(gca,'XColor','k')
    set(gca,'FontSize',12)
    xlim([-2.5 2.5])
    xtickformat('%2.1f');

if  save_fig == 1
    print(gcf,'-dpdf','-fillpage','../Figures/Fig_GAR_inf.pdf');
end

