%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% DYNARE CODE FOR THE MODEL UNDER THE REFERENCE REGIME, i.e. HIGH DELTA
%  William Gatt
%  First version: 22 November 2017
%  Updated for JMCB: 8 June 2021
% 
%  -- used to run model over different parameters

@#define iterate_delta = 1   

% Declare variables and shocks
     @#include "variables_and_shocks.mod"

% Calibrate model 
     @#include "param_calib.mod"
     
     @#if iterate_delta 
        @#include "param_calib_sim.mod"
     @#endif
          
% Declare the model equations
    @#include "model_equations.mod"    

% Add relevant equations to model 
    model;
       [name = 'Notional debt']
       B_NOT = (1-gamma)*(m*N_H_W) + gamma*B_NOT(-1)/PI;
       [name = 'Net housing wealth -- borrowers']
       N_H_W = ((q_1*PI_1)/R)*Hb; 
       [name = 'LM']
       mju = 0;
       [name = 'Leverage limit']
       MaxLev = B - B_NOT;

       [name = 'Macroprudential policy rule']
       m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_mH)*(1-rho_m)))*(m(-1)^(rho_m));
%        m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_mh)*(1-rho_m)))*(m(-1)^(rho_m))*m_shock;       
    end;


% Calibrate shocks
    @#include "shock_calibration.mod"
