%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% Showing IRFs in terms of percentage deviations from baseline
%
% INPUT: List of shocks and endogenous variables to be tracked by DYNARE,
% as well as the IRFs in absolute changes from the steady state and the 
% steady state values of the endogenous variables. 
%
% OUTPUT: A figure showing the IRFs in terms of relative (percentage) or 
% absolute deviations from steady state.
%
% William Gatt
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Option to save output to PDF
 save_fig = 0;  % 0 - No, 1 - Yes
 
 get_IRFS;

%% Generating figures for each shock

for m = 1:n_shocks
    
    
    fig_name = ['Shock to ',exo_list{m},  ' under a Taylor approximation of order ', num2str(options_.order)];
    figure('Name',fig_name,'PaperPositionMode','auto',...
            'PaperOrientation','landscape',...
            'PaperSize',[32 25])
    
    for n = 1:n_endovar
        varname = endo_list{n};      
        zeroline = (zeros(length(struct.irfs.(endo_list{n}).(exo_list{m})),1))';
%       Horizon = (1:length(struct.irfs.(endo_list{n}).(exo_list{m})))/1;  % QUARTERS
        Horizon = (1:length(struct.irfs.(endo_list{n}).(exo_list{m})))/4;  % YEARS
        
        if n_endovar<2
            sp(n)= subplot(1,1,n);
        elseif n_endovar<=2
            sp(n)= subplot(2,1,n);
        elseif n_endovar<=4
            sp(n)= subplot(2,2,n);
        elseif n_endovar<=6 
            sp(n)= subplot(2,3,n);
        elseif n_endovar<=9
            sp(n)= subplot(3,3,n);
        elseif n_endovar<=12
            sp(n)= subplot(4,3,n);
        elseif n_endovar<=16
            sp(n)= subplot(4,4,n);
        elseif n_endovar<=20
            sp(n)= subplot(5,4,n);
        elseif n_endovar<=25
            sp(n)= subplot(5,5,n);
        elseif n_endovar<=30
            sp(n)= subplot(5,6,n);
        elseif n_endovar<=35
            sp(n)= subplot(6,6,n);
        else
%             sp(n) = subplot(floor((n_endovar/7)),ceil(n_endovar/7),n);
            sp(n) = subplot(5,ceil((n_endovar)/5),n);
        end
  
        
        hold on
%         h1 = plot(Horizon, [0 struct.irfs.(endo_list{n}).(exo_list{m})]);
%         h1 = plot(Horizon, [NaN struct.irfs.(endo_list{n}).(exo_list{m})]);

    if max(abs([struct.irfs.(endo_list{n}).(exo_list{m})]) ) < 1e-10  % if response is negligble, don't plot
        h1 = plot(Horizon, zeros(length(struct.irfs.(endo_list{n}).(exo_list{m})),1));
    else
        h1 = plot(Horizon, [struct.irfs.(endo_list{n}).(exo_list{m})]);
    end

        h2 = plot(Horizon, zeroline);
        
%         set(h1,'Color','b','LineWidth',3,'LineStyle','-',...
%         set(h1,'Color',[0 0.6 1],'LineWidth',3,'LineStyle','--')%,...
         set(h1,'Color',[0 0.6 1],'LineWidth',2,'LineStyle','-');
%                'Marker','o','MarkerSize',4,...
%                'MarkerIndices',1:4:length(Horizon));
        set(h2,'Color','k','LineWidth',1.5);
        hold off
        grid on

        xlim([min(Horizon) max(Horizon)]);
        if Horizon(2)<2
            xlabel('years','interpreter','latex');
        else
            xlabel('quarters','interpreter','latex');
        end
%         set(gca,'YTickLabel',num2str(get(gca,'YTick').'))
%         ylim([(min(min(struct.irfs.(endo_list{n}).(exo_list{m})),-0.001)) (max(max(struct.irfs.(endo_list{n}).(exo_list{m})),0.001))]);

        Steady_State = SS.(endo_list{n});
            if Steady_State == 0 
                ylabel(sp(n),'Abs. deviation from SS', 'FontSize', 10,'interpreter','latex');
            else
                if max(strcmpi(endo_list(n),endo_r_list))==1            
                    ylabel(sp(n),'pp. deviation from SS', 'FontSize', 10,'interpreter','latex');
                else
                    ylabel(sp(n),'\% deviation from SS', 'FontSize', 10,'interpreter','latex');
                end
            end 
%          title(strcat('$',{varname},'$'),'FontSize',12,'Interpreter','latex')
         title({varname},'FontSize',12)

        Title = fig_name;
        ha = axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1],'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');
%         text(0.5, , Title,'HorizontalAlignment','center','VerticalAlignment', 'top', 'FontSize', 10);
    end
    
    if save_fig==1
        savename = strcat('IRF_',exo_list{m},'.pdf');
        print(gcf,'-dpdf','-fillpage', savename);
    end
end


%------------------------------------------------



