%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% Set up preamble

% Endogenous variables
var 
    Cs          % Saver consumption    
    B           % loans/deposits
    R           % Gross nominal interest rate
    ws          % Real wage rate for savers
    Ns          % Hours worked by saver
    Cb          % Borrower consumption
    q           % (relative) price of housing
    wb          % Wage rate for borrowers
    Nb          % Hours worked by borrower
    mju         % LM on borrowing constraint
    Y           % Output
    MC          % Real marginal costs
    Hs          % Housing - savers 
    Hb          % Housing - borrowers
    PI          % Gross quarterly inflation 
    Upsilon     % Numerator in price setting
    Phi         % Denominator in price setting
    s           % Price dispersion
    j           % Housing taste shock
    m           % LTV ratio (MP rule)
    ptilde      % relative prices
    N
    
   % Great Ratios
    Cs_Y        % GR - Cs/Y
    Cb_Y        % GR - Cb/Y
    qH_4Y       % GR - Housing wealth /Y
    qHs_4Y      % GR - Housing wealth /Y
    qHb_4Y      % GR - Housing wealth /Y
    B_4Y        % GR - Total loans /Y
    PI_4B       % Credit growth (y-o-y)
    PI_q        % Real House price growth
    Cstilde     % Consumption (with habit persistence)
    Cbtilde     % Consumption (with habit persistence)
    r           % Net nominal interest rate
    r4          % Net nominal interest rate, annualised
    rr          % Net real interest rate
    rr4         % Net real interest rate, annualised
    PI4         % Gross inflation, annualised
    LTI         % Loan to income ratio
    eps_j40
    eps_j41
    eps_j42
    eps_j43
    eps_j44
    eps_ETA0
    eps_ETA1
    eps_ETA2
    eps_ETA3
    eps_ETA4
    PI1
    PI2
    PI3
    nshock
    wNb
    wNs
    q_1
    PI_1
    N_H_W
    B_NOT
    MaxLev
    PI_Y
    PI_Y1
    PI_Y2
    PI_Y3
    PI_Y4
    ;
    

% Exogenous (shock) variables
varexo 

    eps_j       % Housing preference shock
    eps_j4      % Housing preference news shock
    eps_ETA     % Non-fundamental housing shock
    eps_ETA_surprise
;

% Parameters
parameters
    varphi      % Intertemporal elasticity of substitution - leisure     
    tau         % Preference parameter on labour
    j_ss        % Preference parameter on housing in SS
    betas       % Discount factor - savers
    betab       % Discount factor - borrowers
    sigma       % Elasticity of substitution
    m_ss        % SS LTV ratio
    alphaa      % Production parameter
    omega       % Fraction of non-adjusters in t
    delta_Pi    % Policy parameter - inflation
    delta_Y     % Policy parameter - output
    delta_m     % Policy parameter - symmetric reaction 
    delta_mL    % Policy parameter - LTV reaction LOW    
    delta_mH    % Policy parameter - LTV reaction HIGH
    g           % Habits in persistence
    rho_j       % Persistence parameter for j 
    rho_R       % Smoothness in R
    rho_m       % Smoothness in phi (LTV)
    gamma       % Persistence in borrowing (not used)
 ;