clear all
close all

randn('seed',1234567)  
rand('seed' ,1234567)

%% Load VAR Coefficient
load('Bayesian_TIRFs_centered_7Q_deficit_AG_3_3_56')

%% *************************  Set Starting Parameters *************************
random  = 1:draws;               % replications of reduced form errors
R       = 25;                    % replications error-draws
B       = 25;                    % replications history-draws
count_R = 0;                     % set counter for history replications in REC
count_E = 0;                     % set counter for history replications in EXP
s_desc.MA_terms = 4;             % BACKWARD-looking MA 4 quarters
p = s_desc.MA_terms+2;           % length of history: MA_terms: how many observations needed to calculate moving average; +2 because 
                                 % we loose 1 obs. to get growth rate and 1 obs. because z is lagged (z_t-1)  
Nvar = s_desc.Nvar;              % Number of variables
h = nhor+1;                      % Horizon for impulse responses
lags = VARlags000;               % lags in the VAR model
Shockpos = 1;                    % Position of shock in the vector of observations
SR = 5;                          % Number of restricted IRF periods
OmegaE = Omega0E;                % Constant Variance-Covariance Matrix in Expansion Regime
OmegaR = Omega1E;                % Constant Variance-Covariance Matrix in Recession Regime
betaE = beta0E;                  % vector of coefficients: expansion 
betaR = beta1E;                  % vector of coefficients: recession
X_dataR = [];                    % Matrix to store observations belonging to Recession 
X_dataE = [];                    % Matrix to store observations belonging to Expansion
z_mean = mean(Z);                % SAMPLE mean transition variable 
z_std  = std(Z);                 % SAMPLE sta dev transition variable 
X_data = [Y0 (1:size(Y0,1))'];
X_data(:,Nvar+2)  = Z;
size_shock = std(Y0(:,Shockpos));

%%  Storage
CXsim1R_density = NaN(h+2,Nvar,draws);
CXsim2R_density = NaN(h+2,Nvar,draws);
CGirf_R_density = NaN(h+1,Nvar,draws);
Fz1_R_density   = NaN(h+2,draws);
Fz2_R_density   = NaN(h+2,draws);
Z1_R_density    = NaN(h+2,draws);
Z2_R_density    = NaN(h+2,draws);
MultMAX_R_density = NaN(draws,h+1);
MultSUM_R_density = NaN(draws,h+1);

CXsim1E_density = NaN(h+2,Nvar,draws);
CXsim2E_density = NaN(h+2,Nvar,draws);
CGirf_E_density = NaN(h+1,Nvar,draws);
Fz1_E_density   = NaN(h+2,draws);
Fz2_E_density   = NaN(h+2,draws);
Z1_E_density    = NaN(h+2,draws);
Z2_E_density    = NaN(h+2,draws);
MultMAX_E_density = NaN(draws,h+1);
MultSUM_E_density = NaN(draws,h+1);

CXsim1R_density_mean = NaN(h+2,Nvar,draws);
CXsim2R_density_mean = NaN(h+2,Nvar,draws);
CGirf_R_density_mean = NaN(h+1,Nvar,draws);
Fz1_R_density_mean   = NaN(h+2,draws);
Fz2_R_density_mean   = NaN(h+2,draws);
Z1_R_density_mean    = NaN(h+2,draws);
Z2_R_density_mean    = NaN(h+2,draws);
MultMAX_R_density_mean = NaN(draws,h+1);
MultSUM_R_density_mean = NaN(draws,h+1);

CXsim1E_density_mean = NaN(h+2,Nvar,draws);
CXsim2E_density_mean = NaN(h+2,Nvar,draws);
CGirf_E_density_mean = NaN(h+1,Nvar,draws);
Fz1_E_density_mean   = NaN(h+2,draws);
Fz2_E_density_mean   = NaN(h+2,draws);
Z1_E_density_mean    = NaN(h+2,draws);
Z2_E_density_mean    = NaN(h+2,draws);
MultMAX_E_density_mean = NaN(draws,h+1);
MultSUM_E_density_mean = NaN(draws,h+1);

parpool(16)
parfor ii = 1:draws
    %% Preparation
    % DRAW REDUCED FORM COEFFICIENTS
    gamma0 = Gamma(random(ii)); 
    c      = C(random(ii));    
    Fz     = exp(-gamma0/theta*(Z0(:,1)-c))./(1+exp(-gamma0/theta*(Z0(:,1)-c)));
    betaE  = Beta_E(:,:,random(ii));
    betaR  = Beta_R(:,:,random(ii));
    omegaE  = Omega_E(:,:,random(ii));
    omegaR  = Omega_R(:,:,random(ii));
    
    % CALCULATE CONDITIONAL RESIDUALS FROM THE MODEL
    residss1 = NaN(Nvar,s_desc.T); 
    for i=1:s_desc.T
            residss1(:,i)= Y0(i,:)' - (Fz(i,1)*betaR*X(i,:)') - ((1-Fz(i,1)) * betaE*X(i,:)');
    end
    resids=residss1';
    
    % SEPARATE THE DATA INTO Expansionary/recessionary periods
    X_dataR = [];
    X_dataE = [];
    for i = p:s_desc.T
        if  Fz(i)  > 0.8 && Fz(i) < 1  % Recession
            X_dataR = [X_dataR; X_data(i,:)];
        elseif Fz(i) > 0 && Fz(i) < 0.8 % Normal Times
            X_dataE = [X_dataE; X_data(i,:)];
        end
    end            
    
    if size(X_dataR,1) >= 10 && size(X_dataE,1) >= 10
    
    % RANDOMLY DRAW RESIDUALS
    f_errors = NaN(Nvar,h+1,R);
    for jj = 1:R             
        f_errors(:,:,jj) = datasample(resids',h+1,2); 
    end
    
    %% Recessions
    CXsim1R_density0    = zeros(h+2,Nvar,R,B);
    CXsim2R_density0    = zeros(h+2,Nvar,R,B);
    CGirf_R_density0    = zeros(h+1,Nvar,R,B);
    Fz1_R_density0      = NaN(h+2,R,B);
    Fz2_R_density0      = NaN(h+2,R,B);
    Z1_R_density0       = NaN(h+2,R,B);
    Z2_R_density0       = NaN(h+2,R,B);
    MultMAX_R_density0  = NaN(R,B,h+1);
    MultSUM_R_density0  = NaN(R,B,h+1);
    
    count_R = count_R+1;
    X0      = datasample(X_dataR,1);                                       % Draw starting point of history from matrix of Recessionary Observations
    indexR  = X0(1,Nvar+1);
    
    for i=1:p-1                         
        X0   = [X0; X_data(indexR-i,:)];                                   % Draw rest of history from matrix of Recessionary Observations that corresponds to the chosen starting point  
    end
    
    z0      = (X0(1,Nvar+2)-z_mean)/z_std;                                 % Compute z for this particular starting value
    Fz0     = exp(-gamma0/theta*(z0-c))./(1+exp(-gamma0/theta*(z0-c)));    % Compute Fz for this particular starting value
     
    disp(['Initial history ',num2str(ii),' recession: z (standardized) = ',num2str(z0)]); 
    
    Omega_t = Fz0*omegaR+(1-Fz0)*omegaE;
    C_mat = chol(Omega_t,'lower');  
        
    z1R = [z0; zeros(h+1,1)];           % values of z in path with NO shock
    z2R = z1R;                          % values of z in path with SHOCK
    Fz1R= [Fz0; zeros(h+1,1)];          % evolution of F(z) in path with NO shock
    Fz2R= Fz1R;                         % evolution of F(z) in path with SHOCK           
    dY_values1R = [X0(1:p-1,Nvar+2) zeros(p-1,h+1)];   % store values of output growth NO shock
    dY_values2R = dY_values1R;                         % store values of output growth in path with SHOCK
    XsimR = zeros(h+2,lags*Nvar);                        % creates a matrix to store: history + simulations + lags
    
    kk=0;
    for i=1:lags 
        XsimR(1,i+kk:i+kk+Nvar-1)=X0(i,1:Nvar);       
        kk=kk+Nvar-1;                                  
    end
    
    Xsim1R=XsimR;      % --> Will store evolution of variables path with NO shock
    Xsim2R=XsimR;      % --> Will store evolution of variables path with SHOCK
    CXsim1R = XsimR;                       % --> Will store evolution of variables path with NO shock
    CXsim2R = XsimR;                       % --> Will store evolution of variables path with SHOCK
    
    for jj = 1 : R
        rr = 1;
        drawcount=1;
        while rr <= B && drawcount <= 25000
            
            shocks = getShocks(Nvar,f_errors,C_mat,jj,Shockpos,size_shock);
            
            for i = 1 : h
                Xsim1R(i+1,1:Nvar*lags)  = [(Fz1R(i,1)*betaR*Xsim1R(i,:)' + (1-Fz1R(i,1))*betaE*Xsim1R(i,:)' + f_errors(:,i,jj))' Xsim1R(i,1:Nvar) Xsim1R(i,Nvar+1:Nvar*2)];
                Xsim2R(i+1,1:Nvar*lags)  = [(Fz2R(i,1)*betaR*Xsim2R(i,:)' + (1-Fz2R(i,1))*betaE*Xsim2R(i,:)' + shocks(:,i))'   Xsim2R(i,1:Nvar) Xsim2R(i,Nvar+1:Nvar*2)];
                %Transform growth rates back into log levels
                for nn = 1 : length(tcode)
                    if tcode(nn) == 2 || tcode(nn) == 9
                        CXsim1R(i+1,nn) = CXsim1R(i,nn) + Xsim1R(i+1,nn);
                        CXsim2R(i+1,nn) = CXsim2R(i,nn) + Xsim2R(i+1,nn);
                    else
                        CXsim1R(i+1,nn) = Xsim1R(i+1,nn);
                        CXsim2R(i+1,nn) = Xsim2R(i+1,nn);
                    end
                end
                       
                for j=2:p-1                                   
                    dY_values1R(j,i+1)= dY_values1R(j-1,i);
                    dY_values2R(j,i+1)= dY_values2R(j-1,i);
                end
                dY_values1R(1,i+1)= Xsim1R(i+1,Ypos);       
                dY_values2R(1,i+1)= Xsim2R(i+1,Ypos);
            
                % Calculate endogenous Z and F(z)
                z1R(i+1,1)  =   ((mean(dY_values1R(2:end,i+1)))-z_mean)/z_std;  
                z2R(i+1,1)  =   ((mean(dY_values2R(2:end,i+1)))-z_mean)/z_std;             
                Fz1R(i+1,1) =   exp(-gamma0/theta*(z1R(i+1,1)-c))./(1+exp(-gamma0/theta*(z1R(i+1,1)-c)));
                Fz2R(i+1,1) =   exp(-gamma0/theta*(z2R(i+1,1)-c))./(1+exp(-gamma0/theta*(z2R(i+1,1)-c)));          
            end 
            
            CGirf_R_temp = (CXsim2R(2:end,1:Nvar)- CXsim1R(2:end,1:Nvar));
        
            if  all(CGirf_R_temp(1:SR,Gpos) >= 0)  &&...
                all(CGirf_R_temp(1:SR,Tpos) >= 0)  &&...
                all(CGirf_R_temp(1:SR,Ypos) >= 0)  &&...
                all(CGirf_R_temp(1:SR,Ppos) >= 0)  &&...
                all(CGirf_R_temp(1:SR,Dpos) <= 0)  
       
               CXsim1R_density0(:,:,jj,rr) = CXsim1R(:,1:Nvar);
               CXsim2R_density0(:,:,jj,rr) = CXsim2R(:,1:Nvar);
               CGirf_R_density0(:,:,jj,rr) = (CXsim2R(2:end,1:Nvar)- CXsim1R(2:end,1:Nvar));
               Fz1_R_density0(:,jj,rr) = Fz1R;
               Fz2_R_density0(:,jj,rr) = Fz2R;
               Z1_R_density0(:,jj,rr) = z1R;
               Z2_R_density0(:,jj,rr) = z2R;
               
               for ih = 2:h
                    MultMAX_R_density0(jj,rr,ih-1)= (max(CGirf_R_density0(2:ih,Ypos,jj,rr)))*1/GYsc/(max(CGirf_R_density0(2:ih,Gpos,jj,rr)));
                    MultSUM_R_density0(jj,rr,ih-1)= ((sum(CGirf_R_density0(2:ih,Ypos,jj,rr)))/(sum(CGirf_R_density0(2:ih,Gpos,jj,rr))))*1/GYsc;
               end
               
               rr = rr + 1;
               drawcount = drawcount +1;                             
            else 
               drawcount = drawcount +1;
            end      
        end
    end
    
    %% Expansions
    CXsim1E_density0    = zeros(h+2,Nvar,R,B);
    CXsim2E_density0    = zeros(h+2,Nvar,R,B);
    CGirf_E_density0    = zeros(h+1,Nvar,R,B);
    Fz1_E_density0      = NaN(h+2,R,B);
    Fz2_E_density0      = NaN(h+2,R,B);
    Z1_E_density0       = NaN(h+2,R,B);
    Z2_E_density0       = NaN(h+2,R,B);
    MultMAX_E_density0  = NaN(R,B,h+1);
    MultSUM_E_density0  = NaN(R,B,h+1);
    
    count_E = count_E+1;
    X0      = datasample(X_dataE,1);                                       % Draw starting point of history from matrix of Recessionary Observations
    indexE  = X0(1,Nvar+1);
    
    for i=1:p-1                         
        X0   = [X0; X_data(indexE-i,:)];                                   % Draw rest of history from matrix of Recessionary Observations that corresponds to the chosen starting point  
    end
    
    z0      = (X0(1,Nvar+2)-z_mean)/z_std;                                 % Compute z for this particular starting value
    Fz0     = exp(-gamma0/theta*(z0-c))./(1+exp(-gamma0/theta*(z0-c)));    % Compute Fz for this particular starting value
    
    disp(['Initial history ',num2str(ii),' expansion: z (standardized) = ',num2str(z0)]);
    
    Omega_t = Fz0*omegaR+(1-Fz0)*omegaE;
    C_mat = chol(Omega_t,'lower');                    
         
    z1E = [z0; zeros(h+1,1)];           % values of z in path with NO shock
    z2E = z1E;                          % values of z in path with SHOCK
    Fz1E= [Fz0; zeros(h+1,1)];          % evolution of F(z) in path with NO shock
    Fz2E= Fz1E;                         % evolution of F(z) in path with SHOCK           
    dY_values1E = [X0(1:p-1,Nvar+2) zeros(p-1,h+1)];   % store values of output growth NO shock
    dY_values2E = dY_values1E;                         % store values of output growth in path with SHOCK
    XsimE = zeros(h+2,lags*Nvar);                        % creates a matrix to store: history + simulations + lags
    
    kk=0;
    for i=1:lags 
        XsimE(1,i+kk:i+kk+Nvar-1)=X0(i,1:Nvar);       
        kk=kk+Nvar-1;                                  
    end
    
    Xsim1E=XsimE;      % --> Will store evolution of variables path with NO shock
    Xsim2E=XsimE;      % --> Will store evolution of variables path with SHOCK
    CXsim1E = XsimE;                       % --> Will store evolution of variables path with NO shock
    CXsim2E = XsimE;                       % --> Will store evolution of variables path with SHOCK
    
    for jj = 1 : R
        
        rr = 1;
        drawcount=1;
        while rr <= B && drawcount <= 25000
            
            shocks = getShocks(Nvar,f_errors,C_mat,jj,Shockpos,size_shock);
                       
            for i = 1 : h
                Xsim1E(i+1,1:Nvar*lags)  = [(Fz1E(i,1)*betaR*Xsim1E(i,:)' + (1-Fz1E(i,1))*betaE*Xsim1E(i,:)' + f_errors(:,i,jj))' Xsim1E(i,1:Nvar) Xsim1E(i,Nvar+1:Nvar*2)];
                Xsim2E(i+1,1:Nvar*lags)  = [(Fz2E(i,1)*betaR*Xsim2E(i,:)' + (1-Fz2E(i,1))*betaE*Xsim2E(i,:)' + shocks(:,i))'   Xsim2E(i,1:Nvar) Xsim2E(i,Nvar+1:Nvar*2)];
                for nn = 1 : length(tcode)
                    if tcode(nn) == 2 || tcode(nn) == 9
                        CXsim1E(i+1,nn) = CXsim1E(i,nn) + Xsim1E(i+1,nn);
                        CXsim2E(i+1,nn) = CXsim2E(i,nn) + Xsim2E(i+1,nn);
                    else
                        CXsim1E(i+1,nn) = Xsim1E(i+1,nn);
                        CXsim2E(i+1,nn) = Xsim2E(i+1,nn);
                    end
                end 
            
                for j=2:p-1                                   
                    dY_values1E(j,i+1)= dY_values1E(j-1,i);
                    dY_values2E(j,i+1)= dY_values2E(j-1,i);
                end
                dY_values1E(1,i+1)= Xsim1E(i+1,Ypos);       
                dY_values2E(1,i+1)= Xsim2E(i+1,Ypos);     
            
                % Calculate endogenous Z and F(z)
                z1E(i+1,1)  =   ((mean(dY_values1E(2:end,i+1)))-z_mean)/z_std;  
                z2E(i+1,1)  =   ((mean(dY_values2E(2:end,i+1)))-z_mean)/z_std;               
                Fz1E(i+1,1) =   exp(-gamma0/theta*(z1E(i+1,1)-c))./(1+exp(-gamma0/theta*(z1E(i+1,1)-c)));
                Fz2E(i+1,1) =   exp(-gamma0/theta*(z2E(i+1,1)-c))./(1+exp(-gamma0/theta*(z2E(i+1,1)-c))); 
            end
            
            CGirf_E_temp = (CXsim2E(2:end,1:Nvar)- CXsim1E(2:end,1:Nvar));
            
            if  all(CGirf_E_temp(1:SR,Gpos) >= 0)   &&...
                all(CGirf_E_temp(1:SR,Tpos) >= 0)   &&...
                all(CGirf_E_temp(1:SR,Ypos) >= 0)   &&...
                all(CGirf_E_temp(1:SR,Ppos) >= 0)   &&...
                all(CGirf_E_temp(1:SR,Dpos) <= 0)    
       
               CXsim1E_density0(:,:,jj,rr) = CXsim1E(:,1:Nvar);
               CXsim2E_density0(:,:,jj,rr) = CXsim2E(:,1:Nvar);
               CGirf_E_density0(:,:,jj,rr) = (CXsim2E(2:end,1:Nvar)- CXsim1E(2:end,1:Nvar));
               Fz1_E_density0(:,jj,rr) = Fz1E;
               Fz2_E_density0(:,jj,rr) = Fz2E;
               Z1_E_density0(:,jj,rr) = z1E;
               Z2_E_density0(:,jj,rr) = z2E;
               
               for ih = 2:h
                    MultMAX_E_density0(jj,rr,ih-1)= (max(CGirf_E_density0(2:ih,Ypos,jj,rr)))*1/GYsc/(max(CGirf_E_density0(2:ih,Gpos,jj,rr)));
                    MultSUM_E_density0(jj,rr,ih-1)= ((sum(CGirf_E_density0(2:ih,Ypos,jj,rr)))/(sum(CGirf_E_density0(2:ih,Gpos,jj,rr))))*1/GYsc;
               end
               
               rr = rr + 1;
               drawcount = drawcount +1;                             
            else 
               drawcount = drawcount +1;
            end 
        end
    end
 
%% Reshape

    CXsim1R_density0   = reshape(CXsim1R_density0,h+2,Nvar,R*B);
    CXsim2R_density0   = reshape(CXsim2R_density0,h+2,Nvar,R*B);
    CGirf_R_density0   = reshape(CGirf_R_density0,h+1,Nvar,R*B);
    Fz1_R_density0     = reshape(Fz1_R_density0,h+2,R*B);
    Fz2_R_density0     = reshape(Fz2_R_density0,h+2,R*B);
    Z1_R_density0      = reshape(Z1_R_density0,h+2,R*B);
    Z2_R_density0      = reshape(Z2_R_density0,h+2,R*B);
    MultMAX_R_density0 = reshape(MultMAX_R_density0,R*B,h+1);
    MultSUM_R_density0 = reshape(MultSUM_R_density0,R*B,h+1);
    
    CXsim1E_density0   = reshape(CXsim1E_density0,h+2,Nvar,R*B);
    CXsim2E_density0   = reshape(CXsim2E_density0,h+2,Nvar,R*B);
    CGirf_E_density0   = reshape(CGirf_E_density0,h+1,Nvar,R*B);
    Fz1_E_density0     = reshape(Fz1_E_density0,h+2,R*B);
    Fz2_E_density0     = reshape(Fz2_E_density0,h+2,R*B);
    Z1_E_density0      = reshape(Z1_E_density0,h+2,R*B);
    Z2_E_density0      = reshape(Z2_E_density0,h+2,R*B);
    MultMAX_E_density0 = reshape(MultMAX_E_density0,R*B,h+1);
    MultSUM_E_density0 = reshape(MultSUM_E_density0,R*B,h+1);
    
%% SAVE THE MEDIAN/MEAN ESTIMATE
    CXsim1R_density(:,:,ii) = nanmedian(CXsim1R_density0,3);
    CXsim2R_density(:,:,ii) = nanmedian(CXsim2R_density0,3);
    CGirf_R_density(:,:,ii) = nanmedian(CGirf_R_density0,3);
    Fz1_R_density(:,ii)     = nanmedian(Fz1_R_density0,2);
    Fz2_R_density(:,ii)     = nanmedian(Fz2_R_density0,2); 
    Z1_R_density(:,ii)      = nanmedian(Z1_R_density0,2);
    Z2_R_density(:,ii)      = nanmedian(Z2_R_density0,2);
    MultMAX_R_density(ii,:) = nanmedian(MultMAX_R_density0,1);
    MultSUM_R_density(ii,:) = nanmedian(MultSUM_R_density0,1);
    
    CXsim1E_density(:,:,ii) = nanmedian(CXsim1E_density0,3);
    CXsim2E_density(:,:,ii) = nanmedian(CXsim2E_density0,3);
    CGirf_E_density(:,:,ii) = nanmedian(CGirf_E_density0,3);
    Fz1_E_density(:,ii)     = nanmedian(Fz1_E_density0,2);
    Fz2_E_density(:,ii)     = nanmedian(Fz2_E_density0,2);
    Z1_E_density(:,ii)      = nanmedian(Z1_E_density0,2);
    Z2_E_density(:,ii)      = nanmedian(Z2_E_density0,2);
    MultMAX_E_density(ii,:) = nanmedian(MultMAX_E_density0,1);
    MultSUM_E_density(ii,:) = nanmedian(MultSUM_E_density0,1);
    
    CXsim1R_density_mean(:,:,ii) = nanmean(CXsim1R_density0,3);   
    CXsim2R_density_mean(:,:,ii) = nanmean(CXsim2R_density0,3);  
    CGirf_R_density_mean(:,:,ii) = nanmean(CGirf_R_density0,3);   
    Fz1_R_density_mean(:,ii)     = nanmean(Fz1_R_density0,2);   
    Fz2_R_density_mean(:,ii)     = nanmean(Fz2_R_density0,2);    
    Z1_R_density_mean(:,ii)      = nanmean(Z1_R_density0,2);      
    Z2_R_density_mean(:,ii)      = nanmean(Z2_R_density0,2);      
    MultMAX_R_density_mean(ii,:) = nanmean(MultMAX_R_density0,1); 
    MultSUM_R_density_mean(ii,:) = nanmean(MultSUM_R_density0,1); 
    
    CXsim1E_density_mean(:,:,ii) = nanmean(CXsim1E_density0,3);   
    CXsim2E_density_mean(:,:,ii) = nanmean(CXsim2E_density0,3);   
    CGirf_E_density_mean(:,:,ii) = nanmean(CGirf_E_density0,3);   
    Fz1_E_density_mean(:,ii)     = nanmean(Fz1_E_density0,2);     
    Fz2_E_density_mean(:,ii)     = nanmean(Fz2_E_density0,2);     
    Z1_E_density_mean(:,ii)      = nanmean(Z1_E_density0,2);      
    Z2_E_density_mean(:,ii)      = nanmean(Z2_E_density0,2);      
    MultMAX_E_density_mean(ii,:) = nanmean(MultMAX_E_density0,1); 
    MultSUM_E_density_mean(ii,:) = nanmean(MultSUM_E_density0,1); 
   
    end 
       
end

MultMAX_DIFF_density = MultMAX_R_density_mean - MultMAX_E_density_mean;
MultSUM_DIFF_density = MultSUM_R_density_mean - MultSUM_E_density_mean;

%% Save the Results

strA=['Bayesian_GIRFs_SR_centered_7Q_AG' '.mat'];
save(strA)

%% Results

%Multipliers
disp('Multiplier')
qus = [16 50, 84]; %16th, 50th and 84th percentile of Multiplier distribution
period = [5 9 20]; %one year, two years and five years after shock
Multiplier_SR_E_quant      =  squeeze(prctile(MultSUM_E_density_mean(:,period)',qus,2));
Multiplier_SR_R_quant      =  squeeze(prctile(MultSUM_R_density_mean(:,period)',qus,2));
Multiplier_SR_Diff         =  squeeze(prctile(MultSUM_DIFF_density(:,period)',qus,2));
Multiplier_SR_quant        = [Multiplier_SR_E_quant, Multiplier_SR_R_quant, Multiplier_SR_Diff];

disp('SR Multipliers')
disp(Multiplier_SR_quant)   

% F
disp('F After Shock')
fig=figure('Name','F(z)')
subplot(1,2,1)
plot(0:h,squeeze(prctile(Fz2_R_density_mean(1:h+1,:),[16 50 84],2)),'b','Linewidth',2)
xlim([0 h])
ylim([0 1])
title('F(z) after a Government Spending Shock in Passive MP regime')
set(gca,'fontsize',16)
subplot(1,2,2)
plot(0:h,squeeze(prctile(Fz2_E_density_mean(1:h+1,:),[16 50 84],2)),'r','Linewidth',2)
title('F(z) after a Government Spending Shock in Active MP regime')
xlim([0 h])
ylim([0 1])
set(gca,'fontsize',16)











