% Original Caggiano, Castelnuovo, Colombo and Nodari (2015) updated for our
% purposes. 

tic
clear all
close all


%%
load('code_Ramey_3_3.mat')

randn('seed',1234567)  
rand('seed',1234567)


%% *************************  Fix some parameters *************************

Thresh = 0.8;                    % treshold to discriminate btw R&E: value of z STANDARDIZED
B = 1000;                        % replications history-draws (initial conditions)
R = 100;                         % replications error-draws
CIup =95;                        % Confidence Interval upper bound
CIlow = 5;                       % Confidence Interval lower bound
s_desc.MA_terms=4;               % BACKWARD-looking MA 4 quarters
p = s_desc.MA_terms+2;           % length of history: MA_terms: how many observations needed to calculate moving average; +2 because 
                                 % we loose 1 obs. to get growth rate and 1 obs. because z is lagged (z_t-1) 
count_R = 0;                     % set counter for history replications in REC
count_E = 0;                     % set counter for history replications in EXP

Nvar = s_desc.Nvar;              % Number of variables
h = s_desc.irf_hor;              % Horizon for impulse responses
lags = s_desc.VARlags;           % lags in the VAR model
Ypos = 4;                        % Position of log output in the vector of observations
Gpos = 2;                        % Position of log Gov.Spend. in the vector of observations
Shockpos = 1;                    % Position of shock in the vector of observations
OmegaE = Omega0E;                % Constant Variance-Covariance Matrix in Expansion Regime
OmegaR = Omega1E;                % Constant Variance-Covariance Matrix in Recession Regime
betaE = beta0E;                  % vector of coefficients: expansion
betaR = beta1E;                  % vector of coefficients: recession
%beta_c = beta_const;             % intercept terms
X_dataR = [];                    % Matrix to store observations belonging to Recession 
X_dataE = [];                    % Matrix to store observations belonging to Expansion
z_mean = mean(Z);                % SAMPLE mean transition variable 
z_std = std(Z);                  % SAMPLE sta dev transition variable 
gamma0 = -3;

%% CALCULATE RESIDUALS FROM THE MODEL

Fz=exp(gamma0*Z0(:,1))./(1+exp(gamma0*Z0(:,1)));
for i=1:s_desc.T
        resids1(:,i)= Y0(i,:)' - (Fz(i,1)*beta1E*X(i,:)') - ((1-Fz(i,1)) * beta0E*X(i,:)'); %- beta_const
end
resids=resids1';
%%
%% *************************  Rearrange the dataset  **********************

X_data =[Y0 (1:size(Y0,1))'];     
T=size(X_data,1);            

% Calculate growth rate of Y
for i = 1:T-1                  
 X_data(i+1,Nvar+2)= 100*(X_data(i+1,Ypos)- X_data(i,Ypos)); 
 end

% Calculate transition variable         
for i = p:T             
 X_data(i,Nvar+3)   = (mean(X_data(i-s_desc.MA_terms:i-1,Nvar+2))-z_mean); 
 X_data(i,Nvar + 4) =  exp(gamma0*X_data(i,Nvar+3))./(1+exp(gamma0*X_data(i,Nvar+3))); %
 
 
 if X_data(i,Nvar+4) > Thresh                          
     X_dataR = [X_dataR; X_data(i,:)];
 else 
     X_dataE = [X_dataE; X_data(i,:)];
 end
end

GYgirf = mean(exp(Y(:,Ypos))./exp(Y(:,Gpos)));  
size_shock = std(Y(:,Shockpos));                % Set size of the shock
                                

%% *********************** History Draws ********************************
while ((count_R<B) || (count_E<B))
    
%% *********************** Extra Step **********************
% See Koop et al.(1996, p.137) 

% Draw parameters 
    drawpos = max(1,min(round(rand(1,1)*length(beta1matF)),length(beta1matF))); 
% Coefficients:  
    betaRdraw = unvec(beta1matF(drawpos,:)',Nvar,Nvar*lags);                     
    betaEdraw = unvec(beta0matF(drawpos,:)',Nvar,Nvar*lags); 
    %beta_c_draw = beta_constmatF(drawpos,:)';                

% Covariance matrix (both matrices REC and EXP needed to recover Omega_t)
    OmegaRdraw0 = unvech(Omega1mat(drawpos,:)'); 
    OmegaRdraw1 = OmegaRdraw0-triu(OmegaRdraw0,1);       % transform in lower triangular by subtracting upper part
    OmegaRdraw = OmegaRdraw1*OmegaRdraw1';               
    OmegaEdraw0 = unvech(Omega0mat(drawpos,:)'); 
    OmegaEdraw1 = OmegaEdraw0-triu(OmegaEdraw0,1);                 
    OmegaEdraw = OmegaEdraw1*OmegaEdraw1';

%% Step 2: For a given horizon, randomly sample (h+1)xR values of the K-dimensional innovations
 
for jj = 1:R             
f_errors(:,:,jj) = datasample(resids',h+1,2);  
end

%% *********************** Pick a history ********************************
%                       (Step 1 Koop et al.)
%%  RECESSION:
    count_R = count_R+1;
    X0 = datasample(X_dataR,1);  % Draw history from matrix of Recessionary Observations
    indexR = X0(1,Nvar+1);       % Identify the selected observation using index 
               
    for i=1:p-1                         
        X0=[X0; X_data(indexR-i,:)];     
    end                             
  
    %z0 = ((mean(X0(2:p-1,Nvar+2)))-z_mean)/z_std;     % Calculate transition variable 
    %z0 = ((mean(X0(2:p-1,Nvar+2)))-z_mean);            % Calculate transition variable original AG code: z is just demeaned
    %Fz0 = exp(gamma0*z0)./(1+exp(gamma0*z0));  
    
    z0  =   X0(1, Nvar + 3);
    Fz0 =   X0(1, Nvar + 4); 
    
disp(['Initial history ',num2str(count_R),' recession: z (standardized) = ',num2str(z0)]);   

%%
Omega_t = Fz0*OmegaRdraw+(1-Fz0)*OmegaEdraw;
C_mat = chol(Omega_t,'lower');                    
C_matInv=inv(C_mat);  

shocks= f_errors;

% Transform the randomly drawed vector of forecast errors for h=0
% into structural shocks using Omega_t
for jj = 1:R             
shocks(:,1,jj) = (C_matInv)*(shocks(:,1,jj));   
shocks(Shockpos,1,jj)= shocks(Shockpos,1,jj)+size_shock; 
shocks(:,1,jj) = (C_mat)*(shocks(:,1,jj));                
end 

%% 

z1R = [z0; zeros(h+1,1)];           % values of z in path with NO shock
z2R = z1R;                          % values of z in path with SHOCK
Fz1R= [Fz0; zeros(h+1,1)];          % evolution of F(z) in path with NO shock
Fz2R= Fz1R;                         % evolution of F(z) in path with SHOCK           
dY_values1R = [X0(1:p-1,Nvar+2) zeros(p-1,h+1)];   % store values of output growth NO shock
dY_values2R = dY_values1R;                         % store values of output growth in path with SHOCK
XsimR=zeros(h+2,lags*Nvar);                        % creates a matrix to store: history + simulations + lags
j=0;
for i=1:lags 
    XsimR(1,i+j:i+j+Nvar-1)=X0(i,1:Nvar);       
    j=j+Nvar-1;                                  
end
Xsim1R=XsimR;      % --> Will store evolution of variables path with NO shock
Xsim2R=XsimR;      % --> Will store evolution of variables path with SHOCK
 
%%     
%==========================================================================
%                               STEP 3
%               Using the history, simulate the time paths of X_t+h 
%           (If # of lags different from 3, need to change specification)
%==========================================================================
for jj = 1:R 
    for i=1:h+1
        Xsim1R(i+1,1:Nvar*lags)=[(Fz1R(i,1)*betaRdraw*Xsim1R(i,:)'+(1-Fz1R(i,1))*betaEdraw*Xsim1R(i,:)'+f_errors(:,i,jj))' Xsim1R(i,1:Nvar) Xsim1R(i,Nvar+1:Nvar*2)]; %+beta_c_draw
        Xsim2R(i+1,1:Nvar*lags)=[(Fz2R(i,1)*betaRdraw*Xsim2R(i,:)'+(1-Fz2R(i,1))*betaEdraw*Xsim2R(i,:)'+shocks(:,i,jj))' Xsim2R(i,1:Nvar) Xsim2R(i,Nvar+1:Nvar*2)];   %+beta_c_draw
             
        for j=2:p-1                                   
            dY_values1R(j,i+1)= dY_values1R(j-1,i);
            dY_values2R(j,i+1)= dY_values2R(j-1,i);
        end
        dY_values1R(1,i+1)= 100*(Xsim1R(i+1,Ypos)-Xsim1R(i,Ypos));       
        dY_values2R(1,i+1)= 100*(Xsim2R(i+1,Ypos)-Xsim2R(i,Ypos));
        
        % Calculate endogenous Z and F(z) 
        z1R(i+1,1) = ((mean(dY_values1R(2:end,i+1)))-z_mean); %A&G only demean their state variable
        z2R(i+1,1) = ((mean(dY_values2R(2:end,i+1)))-z_mean); %A&G only demean their state variable
        Fz1R(i+1,1) = exp(gamma0*(z1R(i+1,1)))./(1+exp(gamma0*(z1R(i+1,1))));
        Fz2R(i+1,1) = exp(gamma0*(z2R(i+1,1)))./(1+exp(gamma0*(z2R(i+1,1))));
    end
    
    Xsim1R_density0(:,:,jj) = Xsim1R(:,1:Nvar);
    Xsim2R_density0(:,:,jj) = Xsim2R(:,1:Nvar);
    Girf_R_density0(:,:,jj) = (Xsim2R(2:end,1:Nvar)- Xsim1R(2:end,1:Nvar)); 
    Fz1_R_density0(:,jj) = Fz1R;  
    Fz2_R_density0(:,jj) = Fz2R;
    Z1_R_density0(:,jj) = z1R;
    Z2_R_density0(:,jj) = z2R;
        
%----------------  MULTIPLIERS ----------------------------- 
% conditional on 1 history and 1 (out of jj) error dra
   for ih = 2:h+1     
        MultMAX_R_density0(jj,ih-1)= (max(Girf_R_density0(2:ih,Ypos,jj)))*GYgirf/(max(Girf_R_density0(2:ih,Gpos,jj)));
        MultSUM_R_density0(jj,ih-1)= ((sum(Girf_R_density0(2:ih,Ypos,jj)))/(sum(Girf_R_density0(2:ih,Gpos,jj))))*GYgirf; 
    end   
end

%% *************************** SAME STEPS FOR EXPANSION  ********************************
%%  
    count_E = count_E+1;
    X0 = datasample(X_dataE,1);            
    indexE = X0(1,Nvar+1);            
    
    for i=1:p-1                            
        X0=[X0; X_data(indexE-i,:)];        
    end    
        
    z0 =  X0(1, Nvar+3);
    Fz0 =  X0(1, Nvar+4);
 
disp(['Initial history ',num2str(count_E),' expansion: z (standardized) = ',num2str(z0)]);

%%
Omega_t = Fz0*OmegaRdraw+(1-Fz0)*OmegaEdraw;
C_mat = chol(Omega_t,'lower');                    
C_matInv=inv(C_mat);  

shocks= f_errors;

for jj = 1:R             
shocks(:,1,jj) = (C_matInv)*(shocks(:,1,jj));   
shocks(Shockpos,1,jj)= shocks(Shockpos,1,jj)+size_shock; 
shocks(:,1,jj) = (C_mat)*(shocks(:,1,jj));               
end 
           
%%
z1 = [z0; zeros(h+1,1)];           
z2 = z1;                          
Fz1= [Fz0; zeros(h+1,1)];          
Fz2= Fz1;                                   
dY_values1 = [X0(1:p-1,Nvar+2) zeros(p-1,h+1)];   
dY_values2 = dY_values1;                          
Xsim=zeros(h+2,lags*Nvar);                       
j=0;
for i=1:lags 
    Xsim(1,i+j:i+j+Nvar-1)=X0(i,1:Nvar);         
    j=j+Nvar-1;                                     
end
Xsim1=Xsim;      % --> Will store evolution of variables path with NO shock
Xsim2=Xsim;      % --> Will store evolution of variables path with SHOCK
 
%%     
%==========================================================================
%                               STEP 3
%               Using the history, simulate the time paths of X_t+h 
%==========================================================================
for jj = 1:R 
    for i=1:h+1
        Xsim1(i+1,1:Nvar*lags)=[(Fz1(i,1)*betaRdraw*Xsim1(i,:)'+(1-Fz1(i,1))*betaEdraw*Xsim1(i,:)'+f_errors(:,i,jj))' Xsim1(i,1:Nvar) Xsim1(i,Nvar+1:Nvar*2)]; %+beta_c_draw
        Xsim2(i+1,1:Nvar*lags)=[(Fz2(i,1)*betaRdraw*Xsim2(i,:)'+(1-Fz2(i,1))*betaEdraw*Xsim2(i,:)'+shocks(:,i,jj))' Xsim2(i,1:Nvar) Xsim2(i,Nvar+1:Nvar*2)];   %+beta_c_draw
             
        for j=2:p-1                                     
            dY_values1(j,i+1)= dY_values1(j-1,i);
            dY_values2(j,i+1)= dY_values2(j-1,i);
        end
        dY_values1(1,i+1)= 100*(Xsim1(i+1,Ypos)-Xsim1(i,Ypos));          
        dY_values2(1,i+1)= 100*(Xsim2(i+1,Ypos)-Xsim2(i,Ypos));
        
        % Calculate endogenous Z and F(z)
        z1(i+1,1) = ((mean(dY_values1(2:end,i+1)))-z_mean);   %A&G only demean their state variable, use a centered z
        z2(i+1,1) = ((mean(dY_values2(2:end,i+1)))-z_mean);   %A&G only demean their state variable, use a centered z
        Fz1(i+1,1) = exp(gamma0*(z1(i+1,1)))./(1+exp(gamma0*(z1(i+1,1))));
        Fz2(i+1,1) = exp(gamma0*(z2(i+1,1)))./(1+exp(gamma0*(z2(i+1,1))));
    end
    
    Xsim1E_density0(:,:,jj) = Xsim1(:,1:Nvar);
    Xsim2E_density0(:,:,jj) = Xsim2(:,1:Nvar);
    Girf_E_density0(:,:,jj) = (Xsim2(2:end,1:Nvar)- Xsim1(2:end,1:Nvar)); 
    Fz1_E_density0(:,jj) = Fz1;
    Fz2_E_density0(:,jj) = Fz2;
    Z1_E_density0(:,jj) = z1;
    Z2_E_density0(:,jj) = z2;
        
        
%---------------------- MULTIPLIERS ------------------- 
  for ih = 2:h+1     
        MultMAX_E_density0(jj,ih-1)= (max(Girf_E_density0(2:ih,Ypos,jj)))*GYgirf/(max(Girf_E_density0(2:ih,Gpos,jj)));
        MultSUM_E_density0(jj,ih-1)= ((sum(Girf_E_density0(2:ih,Ypos,jj)))/(sum(Girf_E_density0(2:ih,Gpos,jj))))*GYgirf; 
  end   

end
%% *************** SAVE GIRFs and MULTIPLIERs FOR HISTORY 'i'**************     

MultMAX_DIFF_density0 = MultMAX_R_density0 - MultMAX_E_density0; 
MultSUM_DIFF_density0 = MultSUM_R_density0 - MultSUM_E_density0;

   
    Xsim1R_density(:,:,count_R) = median(Xsim1R_density0,3); 
    Xsim2R_density(:,:,count_R) = median(Xsim2R_density0,3); 
    Girf_R_density(:,:,count_R) = median(Girf_R_density0,3);
    Fz1_R_density(:,count_R) = median(Fz1_R_density0,2);   
    Fz2_R_density(:,count_R) = median(Fz2_R_density0,2); 
    Z1_R_density(:,count_R) =  median(Z1_R_density0,2); 
    Z2_R_density(:,count_R) =  median(Z2_R_density0,2); 
    MultMAX_R_density(count_R,:) = median(MultMAX_R_density0,1);
    MultSUM_R_density(count_R,:) = median(MultSUM_R_density0,1);
    
    
    Xsim1E_density(:,:,count_E) = median(Xsim1E_density0,3); 
    Xsim2E_density(:,:,count_E) = median(Xsim2E_density0,3); 
    Girf_E_density(:,:,count_E) = median(Girf_E_density0,3); 
    Fz1_E_density(:,count_E) = median(Fz1_E_density0,2);   
    Fz2_E_density(:,count_E) = median(Fz2_E_density0,2); 
    Z1_E_density(:,count_E) =  median(Z1_E_density0,2);
    Z2_E_density(:,count_E) =  median(Z2_E_density0,2);
    MultMAX_E_density(count_E,:) =  median(MultMAX_E_density0,1);
    MultSUM_E_density(count_E,:) =  median(MultSUM_E_density0,1);    
    

end     

MultMAX_DIFF_density = MultMAX_R_density - MultMAX_E_density;
MultSUM_DIFF_density = MultSUM_R_density - MultSUM_E_density;


%% **********************  END OF REPLICATIONS  **************************

G_CIlowRec = prctile(Girf_R_density,CIlow,3);
G_CIupRec = prctile(Girf_R_density,CIup,3);    
CIlow_multMAX_R = round(prctile(MultMAX_R_density,CIlow,1),2)';
CIup_multMAX_R = round(prctile(MultMAX_R_density,CIup,1),2)';
CIlow_multSUM_R = round(prctile(MultSUM_R_density,CIlow,1),2)';
CIup_multSUM_R = round(prctile(MultSUM_R_density,CIup,1),2)';
CIlow_Fz1_R = prctile(Fz1_R_density,CIlow,2); 
CIup_Fz1_R = prctile(Fz1_R_density,CIup,2);
CIlow_Fz_R = prctile(Fz2_R_density,CIlow,2);  
CIup_Fz_R = prctile(Fz2_R_density,CIup,2);

G_CIlowExp = prctile(Girf_E_density,CIlow,3);  
G_CIupExp = prctile(Girf_E_density,CIup,3);
CIlow_multMAX_E = round(prctile(MultMAX_E_density,CIlow,1),2)';
CIup_multMAX_E = round(prctile(MultMAX_E_density,CIup,1),2)';
CIlow_multSUM_E = round(prctile(MultSUM_E_density,CIlow,1),2)';
CIup_multSUM_E = round(prctile(MultSUM_E_density,CIup,1),2)';
CIlow_Fz1_E = prctile(Fz1_E_density,CIlow,2); 
CIup_Fz1_E = prctile(Fz1_E_density,CIup,2);
CIlow_Fz_E = prctile(Fz2_E_density,CIlow,2); 
CIup_Fz_E = prctile(Fz2_E_density,CIup,2);

    Girf_Rec= median(Girf_R_density,3);  
    MultMAX_R = round(median(MultMAX_R_density,1),2)';
    MultSUM_R = round(median(MultSUM_R_density,1),2)';
    Fz1_R = median(Fz1_R_density,2);   
    Fz_R = median(Fz2_R_density,2); 
    
    Girf_Exp= median(Girf_E_density,3);  
    MultMAX_E = round(median(MultMAX_E_density,1),2)';
    MultSUM_E = round(median(MultSUM_E_density,1),2)';
    Fz1_E = median(Fz1_E_density,2);   
    Fz_E = median(Fz2_E_density,2);

Peak_baseline = [];
Sum_baseline = [];

for i = 4:4:h
Peak_baseline = [Peak_baseline; i MultMAX_E(i,:) CIlow_multMAX_E(i,:) CIup_multMAX_E(i,:)...
                         MultMAX_R(i,:) CIlow_multMAX_R(i,:) CIup_multMAX_R(i,:)];
Sum_baseline = [Sum_baseline; i MultSUM_E(i,:) CIlow_multSUM_E(i,:) CIup_multSUM_E(i,:)...
                     MultSUM_R(i,:) CIlow_multSUM_R(i,:) CIup_multSUM_R(i,:)];   
end

DIFF = squeeze(prctile(MultSUM_DIFF_density, [5 50 94], 1));

strA=['GIRFs_AG_Ramey_' num2str(abs(s_prior.theta0)) '_' num2str(s_desc.VARlags) '.mat'];
save(strA)

toc

%% Results Table 1

% 5th, 50th and 95th percentile of the multiplier distributions for
% expansions and recessions as well as the 5th, 50th and 95th percentiles
% of the distribution of the difference between the two.

Multiplier = [Sum_baseline(5,3), Sum_baseline(5,2), Sum_baseline(5,4),...
              Sum_baseline(5,6), Sum_baseline(5,5), Sum_baseline(5,7),...
              DIFF(1, 20),  DIFF(2, 20),  DIFF(3, 20)];
          
 disp(Multiplier)
