% use news shocks from Ramey (2009)

%function []=code_ramey(theta000,VARlags000)
%clear all
%close all
warning off all

tic
randn('seed',1234567)	% set the seed for random generator

addpath('Functions')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           		PRELIMINARY STEPS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%           Sample
%---------------------------------------------------------------------
s_desc.startS  = 12; 
s_desc.endS    = 252; 
s_desc.sample0 = s_desc.startS:1:s_desc.endS;

%---------------------------------------------------------------------
%            Conventions
%---------------------------------------------------------------------
% Regimes
%  region 1 = recession
%  region 0 = expansion

%---------------------------------------------------------------------
%            Parameters governing the model
%---------------------------------------------------------------------
% VAR parameters
s_desc.VARlags=3;      % number of lags in the VAR
s_desc.Nvar=4;         % number of variables in the VAR
s_desc.irf_hor=20;     % number of periods in the impulse response

s_desc.Gpos=1;         % position of government spending in the vector of variables in the VAR

% Regime specific intercept and trend
s_exo.trendON=0;      % 1 = include a time trend in the VAR;
% 0 = do not include time trend in the VAR

s_exo.trend_regspec=0;  % 1 = allow regime specific time trend
% 0 = do not allow regime specific time trend

s_exo.inter_regspec=0;  % 1 = allow regime specific intercept
% 0 = do not allow regime specific intercept

%---------------------------------------------------------------------
%                   Bayesian VAR parameters
%---------------------------------------------------------------------
s_prior.flagON=0;   % 1 = use a prior in estimation of VAR slopes
% 0 = do not use a prior in estimation of VAR slopes

s_prior.flagOmegaON=0;    % 1 = use a prior in estimation in cov matrix of VAR residuals
% 0 = do not use a prior in cov matrix of VAR
% residuals

s_prior.flagOLS=0;  % 1 = use OLS estimates of the VAR as a prior
% 0 = use Minnesota prior for the VAR (note that MN
% prior does not have a prior for the covariance matrix
% of the residuals)



% penalty for theta which governs the curvature in the transition
% probability function
s_prior.theta0              = -3;    % the slope in the switching equation
s_prior.scale_penalty       = 1e6;
s_prior.scale_penalty_beta  = 0;

% Parameters of the Minnesota prior: random walk for each variable in the VAR
s_prior.lambda_prior=1;   % controls cross-variable responses (smaller values mean stronger prior
s_prior.theta_prior=0.5;  % controls own response of variables

%---------------------------------------------------------------------
%                       MCMC step
%---------------------------------------------------------------------

s_MCMC.runMCMCflag=1; % = 0 do not use MCMC
% = 1 use MCMC

s_MCMC.MCMCn=100000; % number of MCMC draws
s_MCMC.dropfirstT=min(round(s_MCMC.MCMCn*0.3),10000); % burn-in period for MCMC

%---------------------------------------------------------------------
%           			Import data
%---------------------------------------------------------------------
% actual macroeconomic time series: output in data_macro
data_macro_series

% actual series for government revenue and spending: output in data0
data_actual_series

% switching series: output in data_SV
data_switching_series

% News shocks on military spending from Ramey (2009)
data_ramey_0

%---------------------------------------------------------------------
%                    create series for VAR
%---------------------------------------------------------------------

% Core series for the VAR
Y=[data_ramey(s_desc.sample0,6)/100 data0(s_desc.sample0,5) data0(s_desc.sample0,6) data_macro(s_desc.sample0,5) ];

% Switching variable for the VAR
Z=data_SV(s_desc.sample0-1,5); % growth rate of output
% adjust the sqitching variable to have suffiently long recession periods
% (about 1/4 of the sample).
Z0=Z-0.8;

% vector of exogenous variables
s_exo.FX=[data_SV(s_desc.sample0-1,5) data_SV(s_desc.sample0-2,5) data_SV(s_desc.sample0-3,5) data_SV(s_desc.sample0-4,5)];
% s_exo.FX=[data_SV(s_desc.sample0-1,6)];
s_exo.FX=s_exo.FX(s_desc.VARlags+1:end,:);

% number of variables in the vector of forecasts
s_exo.Nforecs=length(s_exo.FX(1,:));

% NEED: introduce the flag for a trend in the VAR
% create the set of basic regressors
X=[];
for i=1:s_desc.VARlags
    X=[X Y(s_desc.VARlags+1-i:end-i,1:s_desc.Nvar)];
end
[rX cX]=size(X);
Y0=Y(s_desc.VARlags+1:end,1:s_desc.Nvar);

Z0=Z0(s_desc.VARlags+1:end,:);

% this is the first order expasion given the guess about theta (the
% curvature in the transition probability)
Xm=[X X.*kron(Z0,ones(1,cX)) X.*kron(Z0.^2.*sign(Z0),ones(1,cX)) ones(length(Y0),1) s_exo.FX];

% NEED: introduce the flag for a trend in the VAR

s_desc.T=length(Y0);    % sample size

s_data.Y0=Y0;
s_data.X=X;
s_data.Z0=Z0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           	run VARs for initial values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%            non-linear VAR to generate residuals
%---------------------------------------------------------------------
% estimate parameters
beta=inv(Xm'*Xm)*(Xm'*Y0);
% compute residuals
resids=Y0-Xm*beta;
% compute the covariance matrix of residuals
Omega=cov(resids);
beta_var=kron(Omega,inv(Xm'*Xm));

s_lag.AICnonlin=log(det(Omega))+2/length(resids)*length(beta);
s_lag.BICnonlin=log(det(Omega))+log(length(resids))/length(resids)*length(beta);

%---------------------------------------------------------------------
%            linear VAR to generate residuals
%---------------------------------------------------------------------
Xm1=[X ones(length(Y0),1) s_exo.FX];
% estimate parameters
betaVARLIN=inv(Xm1'*Xm1)*(Xm1'*Y0);
residsVARLIN=Y0-Xm1*betaVARLIN;
% compute the covariance matrix of residuals
OmegaVARLIN=cov(residsVARLIN);
betaVARLIN=betaVARLIN(1:end-1-s_exo.Nforecs,:);

VARLIN.betaVARLIN=betaVARLIN;
VARLIN.OmegaVARLIN=OmegaVARLIN;
VARLIN.residsVARLIN=residsVARLIN;

s_lag.AIClin=log(det(OmegaVARLIN))+2/length(resids)*length(betaVARLIN);
s_lag.BIClin=log(det(OmegaVARLIN))+log(length(resids))/length(resids)*length(betaVARLIN);

disp('Lag length criteria')
disp(['    Lag = ' num2str(s_desc.VARlags)])
disp('     Non-linear model')
disp(['         AIC =' num2str(s_lag.AICnonlin)])
disp(['         BIC =' num2str(s_lag.BICnonlin)])
disp('     Linear model')
disp(['         AIC =' num2str(s_lag.AIClin)])
disp(['         BIC =' num2str(s_lag.BIClin)])


%---------------------------------------------------------------------
%            generate Minnesota prior
%---------------------------------------------------------------------
if s_prior.flagOLS==0
    [s_prior.prior_mean,s_prior.prior_var]=genMNprior(s_desc.Nvar,s_desc.VARlags,VARLIN.OmegaVARLIN,s_prior.lambda_prior,s_prior.theta_prior);
    
    % note that the MN prior does not provide any information for the
    % covariance matrix of residuals. Hence, impose a flat prior "centered"
    % around the OLS estimate
    
    % covariance matrix of residuals in the VAR
    % compute the covariance matrix for the estimate of OmegaNL
    Dn=dupmat(s_desc.Nvar);
    Dplus=inv(Dn'*Dn)*Dn';
    OmegaVARLIN_var=2*Dplus*kron(OmegaVARLIN,OmegaVARLIN)*Dplus'/s_desc.T;
    
    s_prior.OmegaVAR=1e6*ones(size(diag(OmegaVARLIN_var)));
    s_prior.Omega=OmegaVARLIN;
else
    disp('Use MN prior!')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           NON-Linear VAR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%  First step:
%  estimate the process for the distribution of the error term
%  Omega in recessions and Omega in expansions
%---------------------------------------------------------------------

Omega0=Omega;

% take a small departure for one of the regimes to make sure that starting
% values are slightly different
D=randn(length(Omega));
Omega1=Omega+min(eig(Omega))*D*D';

% Apply Cholesky decomposition since we need to look only for a lower
% trinagular matrix
Omega0chol=chol(Omega0)';
Omega1chol=chol(Omega1)';
s_desc.Omega_length_cov = length(vech(Omega0chol));

% initial parameter values
param0cov=[vech(Omega0chol);  vech(Omega1chol);  s_prior.theta0];

itermax=1000;
disp('fitting first step...')
% I need to pass the prior here
options=optimset('MaxFunEvals',itermax,'MaxIter',itermax,'Display','off','TolFun',1e-6,'TolX',1e-6);
[param01cov,mleLcov,flagquasi]=fminunc(@max_LSTVAR_mleScov,param0cov,options,resids,Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
[param01cov,mleLcov,flagquasi]=fminsearch(@max_LSTVAR_mleScov,param01cov,options,resids,Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
disp(['logL = ' num2str(mleLcov)])

% convert vectorized matrices with parameters into appropriate matrices
[Omega0E,Omega1E,thetaE]=vec2matScov(param01cov,s_desc.Omega_length_cov);

% Omega0E = contemporanoeous covariance in the expansion
% Omega1E = contemporanoeous covariance in the recession

% store the initial values
Omega0E_1=Omega0E;
Omega1E_1=Omega1E;
Omega0chol=chol(Omega0E)';
Omega1chol=chol(Omega1E)';

Omega1chol(4,s_desc.Gpos)=Omega1chol(s_desc.Gpos,s_desc.Gpos)/5.4;
Omega1chol(3,s_desc.Gpos)=-Omega1chol(s_desc.Gpos,s_desc.Gpos)/6;;
Omega1E=Omega1chol*Omega1chol';
Omega1E_1=Omega1E;

Omega0chol(4,s_desc.Gpos)=Omega0chol(s_desc.Gpos,s_desc.Gpos)/5.4/3;
Omega0chol(3,s_desc.Gpos)=Omega0chol(s_desc.Gpos,s_desc.Gpos)/6/3;
Omega0E=Omega0chol*Omega0chol';
Omega0E_1=Omega0E;


param01cov=[vech(Omega0chol);  vech(Omega1chol);  thetaE];

%---------------------------------------------------------------------
%  Second step: ESTIMATE USING MCMC ROUTINES
%  Run the full non-linear model
%  - VAR for each regime
%  - covariance matrix of residuals can vary across regimes.
%            non-linear VAR to generate residuals
%---------------------------------------------------------------------

[Amat,beta0mat,beta1mat,acceptrate,valJ]=...
    optimizeMCMC_struct(param01cov,s_data,s_desc,s_prior,s_exo,s_MCMC);

%---------------------------------------------------------------------
%           Plot impulse responses with confidence intervals
%---------------------------------------------------------------------

% mean values of parameters in the estimated VAR with two regimes
beta1E=unvec(mean(beta1mat(s_MCMC.dropfirstT:end-1,:))',s_desc.Nvar,s_desc.Nvar*s_desc.VARlags);
beta0E=unvec(mean(beta0mat(s_MCMC.dropfirstT:end-1,:))',s_desc.Nvar,s_desc.Nvar*s_desc.VARlags);

[Omega0E,Omega1E,thetaE]=vec2matScov(mean(Amat(s_MCMC.dropfirstT:end-1,:))',s_desc.Omega_length_cov);

Omega_length=length(vech(Omega0E));

Omega1mat=Amat(s_MCMC.dropfirstT:end-1,Omega_length+1:2*Omega_length);
Omega0mat=Amat(s_MCMC.dropfirstT:end-1,1:Omega_length);
beta1matF=beta1mat(s_MCMC.dropfirstT:end-1,:);
beta0matF=beta0mat(s_MCMC.dropfirstT:end-1,:);

% beta0 = dynamics which is the same in expansions
% beta1 = dynamics which is the same in recessions
% Omega0E = contemporanoeous covariance in expansions
% Omega1E = contemporanoeous covariance in recessions

[irf_DimpREC,irf_CImeanREC,CIupREC,CIlowREC,IRFseREC]=irf_VAR_MCMC_struct(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP,irf_CImeanEXP,CIupEXP,CIlowEXP,IRFseEXP]=irf_VAR_MCMC_struct(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);

toc

strA=['code_ramey_' num2str(abs(s_prior.theta0)) '_' num2str(s_desc.VARlags)  '.mat'];
save(strA)

%% Results Table 1

Omega0Ex1=zeros(size(Omega0E));
Omega1Ex1=zeros(size(Omega0E));
for ii=1:length(Amat(s_MCMC.dropfirstT:end-1,:))
    [Omega0Ex,Omega1Ex,thetaEx]=vec2matScov(Amat(s_MCMC.dropfirstT+ii-1,:)',s_desc.Omega_length_cov);
    Omega0Ex1=Omega0Ex1+Omega0Ex;
    Omega1Ex1=Omega1Ex1+Omega1Ex;
end

Omega1E=Omega1Ex1/length(Amat(s_MCMC.dropfirstT:end-1,:));
Omega0E=Omega0Ex1/length(Amat(s_MCMC.dropfirstT:end-1,:));

GYsc=mean(exp(Y(:,4))./exp(Y(:,2)));

% unanticipated shock
s_desc.Gpos=1;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_A,irf_CImeanREC_A,CIupREC_A,CIlowREC_A,IRFseREC_A]=irf_VAR_MCMC_struct_alt(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_A,irf_CImeanEXP_A,CIupEXP_A,CIlowEXP_A,IRFseEXP_A]=irf_VAR_MCMC_struct_alt(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);

% anticipated shock
s_desc.Gpos=2;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_B,irf_CImeanREC_B,CIupREC_B,CIlowREC_B,IRFseREC_B]=irf_VAR_MCMC_struct_alt(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_B,irf_CImeanEXP_B,CIupEXP_B,CIlowEXP_B,IRFseEXP_B]=irf_VAR_MCMC_struct_alt(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);

%%%% MAX
% unanticipated shock
s_desc.Gpos=1;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_A_max,irf_CImeanREC_A_max,CIupREC_A_max,CIlowREC_A_max,IRFseREC_A_max]=irf_VAR_MCMC_struct_alt_max(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_A_max,irf_CImeanEXP_A_max,CIupEXP_A_max,CIlowEXP_A_max,IRFseEXP_A_max]=irf_VAR_MCMC_struct_alt_max(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);

% anticipated shock
s_desc.Gpos=2;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_B_max,irf_CImeanREC_B_max,CIupREC_B_max,CIlowREC_B_max,IRFseREC_B_max]=irf_VAR_MCMC_struct_alt_max(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_B_max,irf_CImeanEXP_B_max,CIupEXP_B_max,CIlowEXP_B_max,IRFseEXP_B_max]=irf_VAR_MCMC_struct_alt_max(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);


%%%% CUMULATIVE
% unanticipated shock
s_desc.Gpos=1;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_A_cum,irf_CImeanREC_A_cum,CIupREC_A_cum,CIlowREC_A_cum,IRFseREC_A_cum]=irf_VAR_MCMC_struct_alt_cum(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_A_cum,irf_CImeanEXP_A_cum,CIupEXP_A_cum,CIlowEXP_A_cum,IRFseEXP_A_cum]=irf_VAR_MCMC_struct_alt_cum(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);

% anticipated shock
s_desc.Gpos=2;         % position of government spending (defense) in the vector of variables in the VAR
[irf_DimpREC_B_cum,irf_CImeanREC_B_cum,CIupREC_B_cum,CIlowREC_B_cum,IRFseREC_B_cum]=irf_VAR_MCMC_struct_alt_cum(beta1E,Omega1E,Omega1mat,beta1matF,s_desc);
[irf_DimpEXP_B_cum,irf_CImeanEXP_B_cum,CIupEXP_B_cum,CIlowEXP_B_cum,IRFseEXP_B_cum]=irf_VAR_MCMC_struct_alt_cum(beta0E,Omega0E,Omega0mat,beta0matF,s_desc);


% scaling multipliers
mmEXP_B=1/sum(irf_CImeanEXP_B(2,:)/20);
mmREC_B=1/sum(irf_CImeanREC_B(2,:)/20);

mmEXP_A=1/sum(irf_CImeanEXP_A(2,:)/20);
mmREC_A=1/sum(irf_CImeanREC_A(2,:)/20);

%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%                   table with multipliers
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

table_for_the_paper_Ramey=[
    max(irf_CImeanEXP_A(4,:)*mmEXP_A*GYsc) (IRFseEXP_A_max(4,:)*mmEXP_A*GYsc) ...
    sum(irf_CImeanEXP_A(4,:)*mmEXP_A*GYsc)/20 (IRFseEXP_A_cum(4,end)*mmEXP_A*GYsc);
    max(irf_CImeanREC_A(4,:)*mmREC_A*GYsc) (IRFseREC_A_max(4,:)*mmREC_A*GYsc) ...
    sum(irf_CImeanREC_A(4,:)*mmREC_A*GYsc)/20 (IRFseREC_A_cum(4,end)*mmREC_A*GYsc)];

% Table 1 Multiplier estimates with the corresponding standard errors for
% recession and expansions.

Multiplier = [table_for_the_paper_Ramey(1,3), table_for_the_paper_Ramey(1,4);...
              table_for_the_paper_Ramey(2,3), table_for_the_paper_Ramey(2,4)];
          
disp(Multiplier)


